<?xml version="1.0" encoding="US-ASCII"?>
<xsl:stylesheet xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
                xmlns:xsd="http://www.w3.org/2001/XMLSchema"
                xmlns:fn="http://www.w3.org/2005/xpath-functions"
                version="2.0">

<!--
     Filters an input schema based on the specification of a customization.

     $Id: filterXSDDK.xsl,v 1.5 2006/12/08 02:58:48 G. Ken Holman Exp $

Copyright (C) - Crane Softwrights Ltd. 
              - http://www.CraneSoftwrights.com/links/res-dev.htm

Redistribution and use in source and binary forms, with or without 
modification, are permitted provided that the following conditions are met:

- Redistributions of source code must retain the above copyright notice, 
  this list of conditions and the following disclaimer. 
- Redistributions in binary form must reproduce the above copyright notice, 
  this list of conditions and the following disclaimer in the documentation 
  and/or other materials provided with the distribution. 
- The name of the author may not be used to endorse or promote products 
  derived from this software without specific prior written permission. 

THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR 
IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES 
OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN 
NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED 
TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR 
PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF 
LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING 
NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, 
EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

Note: for your reference, the above is the "Modified BSD license", this text
      was obtained 2002-12-16 at http://www.xfree86.org/3.3.6/COPYRIGHT2.html#5
-->

<!--an indication of which customization is needed, "strict" or "permitted"-->
<xsl:param name="strictpermitted"/>
<!--the information describing the customization-->
<xsl:param name="componentXML"/>
<!--the identification information to inject into the resulting schemas-->
<xsl:param name="identificationString"/>
<!--the copyright information to inject into the resulting schemas-->
<xsl:param name="copyrightFile"/>
<!--the regular expression matching the start of customiztaion columns-->
<xsl:param name="regexForeign"/>

<!--indent the result-->
<xsl:output indent="yes"/>

<!--this cleans up the output when using indent="yes"-->
<xsl:template match="text()[not(normalize-space())]"/>

<!--identity for all other nodes not matched by the logic below-->
<xsl:template match="@*|node()">
  <xsl:copy>
    <xsl:apply-templates select="@*|node()"/>
  </xsl:copy>
</xsl:template>

<!--an indication that there are multiple declarations-->
<xsl:variable name="one-of-multiple" select="count(/*/xsd:element[@name])!=1"/>

<!--the usage string based on the type of customization-->
<xsl:variable name="usage" 
              select="if ($strictpermitted eq 'strict') then 'USED' else ''"/>

<!--all abie definitions in customization file-->
<xsl:key name="abie" match="ABIE" use="concat(Item[1]/UBLName,'Type')"/>

<!--all elements that are used in this customization-->
<xsl:key name="used"
         match="Item[*[fn:matches(local-name(.),concat($regexForeign,'Usage'))]
                      [.='USED' or .=$usage]]"
         use="concat( ../Item[1]/UBLName,ComponentType,' ',UBLName,'Type' )"/>

<!--open remote file for component definition-->
<xsl:variable name="comp" select="document($componentXML,/)"/>

<!--========================================================================-->
<!--getting started-->

<xsl:template match="/">
  <!--check for type of pruning-->
  <xsl:if test="$strictpermitted!='strict' and $strictpermitted!='permitted'">
    <xsl:message terminate="yes">
      <xsl:text>The value for $strictpermitted can only be 'strict' </xsl:text>
      <xsl:text>or 'permitted'.</xsl:text>
    </xsl:message>
  </xsl:if>
  <!--check there is a customization specification-->
  <xsl:if test="not( $componentXML )">
    <xsl:message terminate="yes">
      <xsl:text>Missing $componentXML customization specification: </xsl:text>
      <xsl:value-of select="$componentXML"/>
    </xsl:message>
  </xsl:if>
  <!--check that there are customization specification columns present-->
  <xsl:for-each select="$comp">
    <xsl:if test="not( $regexForeign ) or not( /*/*/Item/*
                      [fn:matches(local-name(.),concat($regexForeign,'Usage'))]
                  ) or not( /*/*/Item/*
            [fn:matches( local-name(.),concat($regexForeign,'Cardinality'))])">
      <xsl:message terminate="yes">
        <xsl:text>Missing one or both customization specification </xsl:text>
        <xsl:text>columns starting with $regexForeign value of: </xsl:text>
        <xsl:value-of select="$regexForeign"/>
      </xsl:message>
    </xsl:if>
  </xsl:for-each>
  <!--only add identification string if it has been specified-->
  <xsl:if test="$identificationString">
    <xsl:text>
</xsl:text>
    <xsl:comment>
      <xsl:text>
</xsl:text>
      <xsl:value-of select="translate($identificationString,'&#xd;','')"/>
      <xsl:text>
</xsl:text>
    </xsl:comment>
  </xsl:if>
  <!--only add a comment if the copyright file has been specified-->
  <xsl:if test="$copyrightFile">
    <xsl:text>
</xsl:text>
    <xsl:comment>
      <xsl:text>
</xsl:text>
   <xsl:value-of select="translate(unparsed-text($copyrightFile),'&#xd;','')"/>
      <xsl:text>
</xsl:text>
    </xsl:comment>
  </xsl:if>

  <xsl:apply-templates/>
</xsl:template>

<!--========================================================================-->
<!--check an ABIE for being empty or not-->

<xsl:template match="xsd:complexType/xsd:sequence">
  <xsl:variable name="abie" select="key( 'abie', ../@name, $comp )"/>
  <xsl:variable name="childrenUsage" select="$abie/
             Item/*[fn:matches(local-name(.),concat($regexForeign,'Usage'))]"/>
  <xsl:choose>
    <xsl:when test="not( $childrenUsage[ .='USED' or .=$usage ] )">
      <!--then the new type is empty-->
      <xsl:comment>
        <xsl:call-template name="preserve"/>
      </xsl:comment>
    </xsl:when>
    <xsl:otherwise>
      <!--preserve the sequencing of children-->
      <xsl:copy>
        <xsl:apply-templates select="@*|node()">
          <xsl:with-param name="childrenUsage" select="$childrenUsage"
                          tunnel="yes"/>
        </xsl:apply-templates>
      </xsl:copy>
    </xsl:otherwise>
  </xsl:choose>
</xsl:template>

<!--check a BBIE or an ASBIE for being used or not-->
<xsl:template match="xsd:element[@ref]">
  <xsl:param name="childrenUsage" tunnel="yes"/>
  <xsl:variable name="thisElement"
                select="$childrenUsage[ .='USED' or .=$usage ]/
                        ..[current()/@ref = UBLName or
                           ends-with(current()/@ref,concat(':',UBLName))]"/>
  <xsl:choose>
    <xsl:when test="not( $thisElement )">
      <!--then the new element is not used-->
      <xsl:comment>
        <xsl:call-template name="preserve"/>
      </xsl:comment>
    </xsl:when>
    <xsl:otherwise>
      <!--preserve this element use-->
      <xsl:copy>
        <xsl:apply-templates select="@*">
          <xsl:with-param name="childrenUsage" tunnel="yes"/>
        </xsl:apply-templates>
        <xsl:call-template name="handle-cardinality">
          <xsl:with-param name="thisElement" select="$thisElement"/>
        </xsl:call-template>
        <xsl:apply-templates select="node()">
          <xsl:with-param name="childrenUsage" tunnel="yes"/>
        </xsl:apply-templates>
      </xsl:copy>
    </xsl:otherwise>
  </xsl:choose>
</xsl:template>

<!--determine if new cardinality is valid and different from old cardinality-->
<xsl:template name="handle-cardinality">
  <xsl:param name="thisElement"/>
  <xsl:variable name="cardinality" 
                select="$thisElement/*[fn:matches(local-name(.),
                                       concat($regexForeign,'Cardinality'))]"/>
  <xsl:variable name="min" 
    select="if ( $cardinality='1' ) then '1' 
                                    else substring-before($cardinality,'..')"/>
  <xsl:variable name="max"
    select="if ( $cardinality='1' ) then '1' 
                                    else substring-after($cardinality,'..')"/>
  <xsl:choose>
    <xsl:when test="not(normalize-space($cardinality))">
      <!--cannot be empty-->
      <xsl:message terminate="yes">
        <xsl:text>Missing value of cardinality for </xsl:text>
        <xsl:value-of select="$thisElement/UBLName"/>
      </xsl:message>
    </xsl:when>
    <xsl:when test="$cardinality='1' or
                    ( contains($cardinality,'..')
                      and $min castable as xsd:decimal
                      and ( $max castable as xsd:decimal or $max = 'n' ) )">
      <!--at least it is expressed suitably-->
      <xsl:choose>
        <xsl:when test="@minOccurs &lt;= $min and
                        $min &lt;= $max and number($max) != 0 and
                        ( $max = 'n' or $max &lt;= @maxOccurs )">
          <!--and it apears to be valid-->
          <xsl:variable name="maxOccurs" 
                       select="if ( $max = 'n' ) then 'unbounded' else $max"/>
          <xsl:attribute name="maxOccurs" select="$maxOccurs"/>
          <xsl:attribute name="minOccurs" select="$min"/>
          <xsl:if test="@minOccurs != $min or @maxOccurs != $maxOccurs">
            <!--document the difference-->
            <xsl:comment>
              <xsl:text>minOccurs="</xsl:text>
              <xsl:value-of select="@minOccurs"/>" maxOccurs="<xsl:text/>
              <xsl:value-of select="@maxOccurs"/>"<xsl:text/>
            </xsl:comment>
          </xsl:if>
        </xsl:when>
        <xsl:otherwise>
          <!--the values supplied are unacceptable-->
          <xsl:message terminate="yes">
            <xsl:text>Improper value of cardinality for </xsl:text>
        <xsl:value-of select="concat($thisElement/UBLName,': ',$cardinality)"/>
      </xsl:message>
        </xsl:otherwise>
      </xsl:choose>
    </xsl:when>
    <xsl:otherwise>
      <!--something wrong with the expression of the cardinality-->
      <xsl:message terminate="yes">
        <xsl:text>Invalid value of cardinality for </xsl:text>
        <xsl:value-of select="concat($thisElement/UBLName,': ',$cardinality)"/>
      </xsl:message>
    </xsl:otherwise>
  </xsl:choose>
</xsl:template>

<!--========================================================================-->
<!--mimic original markup in a comment-->

<xsl:template name="preserve">
  <!--copy indentation-->
  <xsl:copy-of select="preceding-sibling::node()[1][self::text()]"/>
  <xsl:apply-templates select="." mode="preserve"/>
  <!--copy indentation-->
  <xsl:copy-of select="preceding-sibling::node()[1][self::text()]"/>
</xsl:template>

<xsl:template match="*" mode="preserve">
  <!--convert element and attributes to text for comment-->
  <xsl:value-of select="concat('&lt;',name(.))"/>
  <xsl:for-each select="@*">
    <xsl:value-of select="concat(' ',name(.),'=&#34;',.,'&#34;')"/>
  </xsl:for-each>
  <xsl:text>></xsl:text>
  <xsl:apply-templates mode="preserve"/>
  <xsl:value-of select="concat('&lt;/',name(.),'>')"/>
</xsl:template>

</xsl:stylesheet>
